#ifndef _ULAN_H
#define _ULAN_H

#include <ul_lib/ul_drvdef.h>

#ifdef _WIN32

 #define UL_DEV_NAME "\\\\.\\UL_DRV"

 typedef HANDLE ul_fd_t;
 #define UL_FD_INVALID INVALID_HANDLE_VALUE
 typedef size_t ssize_t;

 #define WITHOUT_SYS_SELECT
 #define HAS_GETOPT_LONG

#elif defined(__DJGPP__)

 #define UL_DEV_NAME "1"

 struct ul_opdata;
 typedef struct ul_opdata *ul_fd_t;
 #define UL_FD_INVALID (NULL)

 #define WITHOUT_SYS_SELECT
 #define UL_DRV_IN_LIB

#else /* !_WIN32 && !__DJGPP__ */

 #define UL_DEV_NAME "/dev/ulan"

 typedef int ul_fd_t;
 #define UL_FD_INVALID (-1)

 #define HAS_GETDELIM
 #define HAS_GETOPT_LONG

#endif /* _WIN32 */

/* driver interface */

ul_fd_t	ul_open(const char *dev_name, const char *options);
int	ul_close(ul_fd_t ul_fd);
ssize_t	ul_read(ul_fd_t ul_fd, void *buffer, size_t size);
ssize_t	ul_write(ul_fd_t ul_fd, const void *buffer, size_t size);
int	ul_newmsg(ul_fd_t ul_fd,const ul_msginfo *msginfo);
int	ul_tailmsg(ul_fd_t ul_fd,const ul_msginfo *msginfo);
int	ul_freemsg(ul_fd_t ul_fd);
int	ul_acceptmsg(ul_fd_t ul_fd,ul_msginfo *msginfo);
int	ul_actailmsg(ul_fd_t ul_fd,ul_msginfo *msginfo);
int	ul_addfilt(ul_fd_t ul_fd,const ul_msginfo *msginfo);
int	ul_abortmsg(ul_fd_t ul_fd);
int	ul_rewmsg(ul_fd_t ul_fd);
int	ul_inepoll(ul_fd_t ul_fd);
int	ul_drv_debflg(ul_fd_t ul_fd,int debug_msk);
int	ul_fd_wait(ul_fd_t ul_fd, int wait_sec);

/* simple message operations */

int ul_send_command(ul_fd_t ul_fd,int dadr,int cmd,int flg,
                    const void *buf,int len);
int ul_send_command_wait(ul_fd_t ul_fd,int dadr,int cmd,int flg,
                         const void *buf,int len);
int ul_send_query(ul_fd_t ul_fd,int dadr,int cmd,int flg,
                  const void *buf,int len);
int ul_send_query_wait(ul_fd_t ul_fd,int dadr,int cmd,int flg,
		       const void *bufin,int lenin,void **bufout,int *lenout);

/* basic uLan commands/services */

#define UL_CMD_OISV	0x10	/* Object Interface Service */
#define UL_CMD_LCDABS	0x4f	/* Absorbance data block */
#define UL_CMD_LCDMRK	0x4e	/* Mark */
#define UL_CMD_NCS	0x7f	/* Network Control Service */

/* definitions of basic uLan OI commands */

#define ULOI_AOID   10	/* name is defined in ASCII for DOIx */
#define ULOI_DOII   12	/* description of input objects */
#define ULOI_DOIO   14	/* description of output objects */
#define ULOI_QOII   16	/* ID numbers of recognized input objects */
#define ULOI_QOIO   18	/* ID numbers of recognized output objects */
#define ULOI_RDRQ   20	/* object values read request */
#define ULOI_STATUS 30	/* read instrument status */
#define ULOI_ERRCLR 31	/* clear error status */

typedef struct uloi_coninfo {
  int adr;	/* address of target module */
  int cmd;	/* service/cmd number for uLOI on target */
  int bcmd;	/* service/cmd number for returned messages */
  int sn;	/* sequence counter */
  int bsn;	/* sequence counter of target module */
  int outflg;	/* flags used for outgoing messages */
  ul_fd_t ul_fd;/* handle for ul_drv */
  int timeout;	/* timeout */
} uloi_coninfo;

/* definitions of basic uLan OI functions */

int uloi_debug_flg;

uloi_coninfo* uloi_open(char *ul_dev_name,int adr,int cmd,
			int bcmd, int timeout);
void uloi_close(uloi_coninfo *coninfo);
int uloi_transfer(uloi_coninfo *coninfo,
		  uchar *bufin,int lenin,uchar **bufout,int *lenout);
int uloi_set_var(uloi_coninfo *coninfo,int oid, void *val, int size);
int uloi_get_var(uloi_coninfo *coninfo,int oid, void *val, int size);
int uloi_set_var_u2(uloi_coninfo *coninfo,int oid,unsigned val);
int uloi_get_var_u2(uloi_coninfo *coninfo,int oid,unsigned *val);
int uloi_send_cmd(uloi_coninfo *coninfo,int oid);
int uloi_get_oids(uloi_coninfo *coninfo,int list,int **oids_list);
int uloi_get_oiddes(uloi_coninfo *coninfo,int list, int oid);
int uloi_get_aoiddes(uloi_coninfo *coninfo,int list,char *aoid);

/* UL_CMD_NCS	Network Control Service */

#define ULNCS_SET_ADDR	0xC1	/* SN0 SN1 SN2 SN3 NEW_ADR */
#define ULNCS_SID_RQ	0xC2	/* send serial num and ID string request */
#define ULNCS_SID_RPLY	0xC3	/* SN0 SN1 SN2 SN3 ID ... */

#if 0
  #define  read		error_to_use_read
  #define  write	error_to_use_write
  #define  open		error_to_use_open
  #define  close	error_to_use_close
  #define  ioctl	error_to_use_ioctl
#endif

#endif /* _ULAN_H */




